//
//  RWTRefreshView.m
//  PullToRefresh
//
//  Created by Main Account on 5/1/14.
//  Copyright (c) 2014 Razeware LLC. All rights reserved.
//

#import "RWTRefreshView.h"
#import "RWTRefreshItem.h"

static CGFloat kSceneHeight = 120.f;

@interface RWTRefreshView()
@property (assign, nonatomic) CGFloat progress;
@property (strong, nonatomic) NSMutableArray *refreshItems;
@property (assign, nonatomic) BOOL refreshing;
@property (weak, nonatomic) UIScrollView *scrollView;

@property (strong, nonatomic) UIImageView *ground;
@property (strong, nonatomic) UIImageView *buildings;
@property (strong, nonatomic) UIImageView *sun;
@property (strong, nonatomic) UIImageView *capeBack;
@property (strong, nonatomic) UIImageView *cat;
@property (strong, nonatomic) UIImageView *capeFront;

@property (strong, nonatomic) UIImageView *sign;
@property (strong, nonatomic) RWTRefreshItem *readyItem;
@property (assign, nonatomic) BOOL showingReadyItem;

@end

@implementation RWTRefreshView

- (id)initWithFrame:(CGRect)frame scrollView:(UIScrollView *)scrollView
{
  self = [super initWithFrame:frame];
  if (self) {
    self.progress = 0;
    self.refreshItems = [NSMutableArray array];
    self.scrollView = scrollView;
    
    [self refreshColor];
    [self setupItems];
  }
  return self;
}

- (void)setupItems {
  self.ground = [[UIImageView alloc] initWithImage:[UIImage imageNamed:@"ground"]];
  self.buildings = [[UIImageView alloc] initWithImage:[UIImage imageNamed:@"buildings"]];
  self.sun = [[UIImageView alloc] initWithImage:[UIImage imageNamed:@"sun"]];
  self.capeBack = [[UIImageView alloc] initWithImage:[UIImage imageNamed:@"cape_back"]];
  self.cat = [[UIImageView alloc] initWithImage:[UIImage imageNamed:@"cat"]];
  self.capeFront = [[UIImageView alloc] initWithImage:[UIImage imageNamed:@"cape_front"]];

  [self addRefreshItem:[[RWTRefreshItem alloc] initWithView:self.buildings centerEnd:CGPointMake(CGRectGetMidX(self.bounds), CGRectGetHeight(self.bounds) - CGRectGetHeight(self.ground.bounds) - CGRectGetHeight(self.buildings.bounds)/2) parallaxRatio:1.5f sceneHeight:kSceneHeight]];

  [self addRefreshItem:[[RWTRefreshItem alloc] initWithView:self.ground centerEnd:CGPointMake(CGRectGetMidX(self.bounds), CGRectGetHeight(self.bounds) - CGRectGetHeight(self.ground.bounds)/2) parallaxRatio:0.5f sceneHeight:kSceneHeight]];
  
  [self addRefreshItem:[[RWTRefreshItem alloc] initWithView:self.sun centerEnd:CGPointMake(CGRectGetWidth(self.bounds) * 0.1, CGRectGetHeight(self.bounds) - CGRectGetHeight(self.ground.bounds) - CGRectGetHeight(self.sun.bounds)) parallaxRatio:3.f sceneHeight:kSceneHeight]];
  
  [self addRefreshItem:[[RWTRefreshItem alloc] initWithView:self.capeBack centerEnd:CGPointMake(CGRectGetMidX(self.bounds), CGRectGetHeight(self.bounds) - CGRectGetHeight(self.ground.bounds)/2 - CGRectGetHeight(self.capeBack.bounds)/2) parallaxRatio:-1.f sceneHeight:kSceneHeight]];

  [self addRefreshItem:[[RWTRefreshItem alloc] initWithView:self.cat centerEnd:CGPointMake(CGRectGetMidX(self.bounds), CGRectGetHeight(self.bounds) - CGRectGetHeight(self.ground.bounds)/2 - CGRectGetHeight(self.cat.bounds)/2) parallaxRatio:1.0f sceneHeight:kSceneHeight]];
  
  [self addRefreshItem:[[RWTRefreshItem alloc] initWithView:self.capeFront centerEnd:CGPointMake(CGRectGetMidX(self.bounds), CGRectGetHeight(self.bounds) - CGRectGetHeight(self.ground.bounds)/2 - CGRectGetHeight(self.capeFront.bounds)/2) parallaxRatio:-1.f sceneHeight:kSceneHeight]];
  
  self.sign = [[UIImageView alloc] initWithImage:[UIImage imageNamed:@"sign"]];
  self.readyItem = [[RWTRefreshItem alloc] initWithView:self.sign centerEnd:CGPointMake(CGRectGetMidX(self.bounds), CGRectGetHeight(self.bounds) - CGRectGetHeight(self.sign.bounds)/2) parallaxRatio:0.5f sceneHeight:kSceneHeight];
  [self addSubview:self.readyItem.view];

 
}

- (void)addRefreshItem:(RWTRefreshItem *)item {
  [self addSubview:item.view];
  [self.refreshItems addObject:item];
}

- (void)refreshColor {
  CGFloat value = (0.7*self.progress) + 0.2;
  self.backgroundColor = [UIColor colorWithRed:value green:value blue:value alpha:1.0];
}

- (void)endRefreshing {
  
  [UIView animateWithDuration:0.4f delay:0.f options:UIViewAnimationOptionCurveEaseOut animations:^{
    UIEdgeInsets newInsets = self.scrollView.contentInset;
    newInsets.top -= kSceneHeight;
    [self.scrollView setContentInset:newInsets];
    
  } completion:^(BOOL finished) {
    self.refreshing = NO;
  }];
  
}

#pragma mark - UIScrollViewDelegate

- (void)scrollViewDidScroll:(UIScrollView *)scrollView {
    
  if (self.refreshing) return;
  
  CGFloat offsetY = MAX(-(scrollView.contentOffset.y + scrollView.contentInset.top), 0);
  self.progress = MIN(MAX(offsetY / kSceneHeight, 0.f), 1.f);
  [self refreshColor];
  for (RWTRefreshItem *item in self.refreshItems) {
    [item centerForProgress:self.progress];
  }
  
  if (self.progress >= 1.0) {
    [self showReadyItem:YES];
  } else {
    [self showReadyItem:NO];
  }
  
}

- (void)scrollViewWillEndDragging:(UIScrollView *)scrollView withVelocity:(CGPoint)velocity targetContentOffset:(inout CGPoint *)targetContentOffset {
  
  if (self.progress >= 1.f) {    
    if (self.delegate && [self.delegate respondsToSelector:@selector(refreshViewDidRefresh:)]) {
      [self beginRefreshing];
      *targetContentOffset = CGPointMake(0, -self.scrollView.contentInset.top);
      [self.delegate refreshViewDidRefresh:self];
    }
  }
}

- (void)beginRefreshing {
  self.refreshing = YES;
  
  [UIView animateWithDuration:0.4f delay:0.f options:UIViewAnimationOptionCurveEaseOut animations:^{
    
    UIEdgeInsets newInsets = self.scrollView.contentInset;
    newInsets.top += kSceneHeight;
    [self.scrollView setContentInset:newInsets];
    
  } completion:^(BOOL finished) {
  }];
  
  [self showReadyItem:NO];
  
}

- (void)showReadyItem:(BOOL)show {
  if (self.showingReadyItem == show) return;
  
  self.showingReadyItem = show;
  [UIView animateWithDuration:0.2 delay:0 options:UIViewAnimationOptionCurveEaseOut|UIViewAnimationOptionBeginFromCurrentState animations:^{
    [self.readyItem centerForProgress:show ? 1.f : 0.f];
  } completion:nil];
}

@end
